@@ -183,6 +183,7 @@ void config_destroy(void)
     if (ctx.sections) {
         free(ctx.sections);
         ctx.sections = NULL;
+        ctx.num_sections = 0;
     }
 }
 
@@ -228,40 +229,44 @@ bool config_command(const char* cmd)
     char section[32];
     char key[32];
     const char* value;
-    char* ptr;
-    const char* it = cmd;
+    const char* ptr;
+    struct str_slice slices[2];
+    size_t size;
 
-    // get section name
-    ptr = section;
-    while (*it != '.') {
-        if (!*it || ptr >= section + sizeof(section) - 1) {
-            goto format_error;
-        }
-        *ptr++ = *it++;
+    // split section.key and value
+    size = str_split(cmd, '=', slices, ARRAY_SIZE(slices));
+    if (size <= 1) {
+        return false;
     }
-    *ptr = 0; // last null
-    ++it;     // skip delimiter
 
-    // get key
-    ptr = key;
-    while (*it != '=') {
-        if (!*it || ptr >= key + sizeof(key) - 1) {
-            goto format_error;
+    // split section and key
+    ptr = slices[0].value + slices[0].len;
+    while (*ptr != '.') {
+        if (--ptr < cmd) {
+            return false;
         }
-        *ptr++ = *it++;
     }
-    *ptr = 0; // last null
-    ++it;     // skip delimiter
 
-    // get value
-    value = it;
+    // section name
+    size = ptr - slices[0].value;
+    if (size > sizeof(section) - 1) {
+        size = sizeof(section) - 1;
+    }
+    memcpy(section, slices[0].value, size);
+    section[size] = 0;
+
+    // key name
+    ++ptr; // skip dot
+    size = slices[0].len - (ptr - slices[0].value);
+    if (size > sizeof(key) - 1) {
+        size = sizeof(key) - 1;
+    }
+    memcpy(key, ptr, size);
+    key[size] = 0;
+
+    value = slices[1].value;
 
-    // load setting
     return config_set(section, key, value) == cfgst_ok;
-
-format_error:
-    fprintf(stderr, "Invalid format: \"%s\"\n", cmd);
-    return false;
 }
 
 void config_add_loader(const char* section, config_loader loader)
