@@ -118,9 +118,11 @@ static void add_dir(const char* dir, bool recursive)
         len += strlen(dir_entry->d_name) + 1 /*last null*/;
         path = malloc(len);
         if (path) {
+            // NOLINTBEGIN(clang-analyzer-security.insecureAPI.strcpy)
             strcpy(path, dir);
             strcat(path, "/");
             strcat(path, dir_entry->d_name);
+            // NOLINTEND(clang-analyzer-security.insecureAPI.strcpy)
 
             if (stat(path, &file_stat) == 0) {
                 if (S_ISDIR(file_stat.st_mode)) {
@@ -148,6 +150,10 @@ static size_t next_entry(size_t start, bool forward)
 {
     size_t index = start;
 
+    if (start == IMGLIST_INVALID) {
+        return image_list_first();
+    }
+
     while (true) {
         if (forward) {
             if (++index >= ctx.size) {
@@ -189,6 +195,10 @@ static size_t next_dir(size_t start, bool forward)
     size_t cur_len;
     size_t index = start;
 
+    if (start == IMGLIST_INVALID) {
+        return image_list_first();
+    }
+
     // directory part of the current file path
     cur_len = strlen(cur_path) - 1;
     while (cur_len && cur_path[cur_len] != '/') {
@@ -369,6 +379,49 @@ size_t image_list_find(const char* source)
     return IMGLIST_INVALID;
 }
 
+size_t image_list_distance(size_t start, size_t end)
+{
+    size_t index = start;
+    size_t step = 0;
+
+    if (start > end) {
+        return IMGLIST_INVALID;
+    }
+
+    while (index != IMGLIST_INVALID && index < end) {
+        ++step;
+        index = next_entry(index, true);
+    }
+
+    return step;
+}
+
+size_t image_list_back(size_t start, size_t distance)
+{
+    size_t index = start;
+    while (distance--) {
+        const size_t next = next_entry(index, false);
+        if (next == IMGLIST_INVALID || next >= start) {
+            break;
+        }
+        index = next;
+    }
+    return index;
+}
+
+size_t image_list_forward(size_t start, size_t distance)
+{
+    size_t index = start;
+    while (distance--) {
+        const size_t next = next_entry(index, true);
+        if (next == IMGLIST_INVALID || next <= start) {
+            break;
+        }
+        index = next;
+    }
+    return index;
+}
+
 size_t image_list_next_file(size_t start)
 {
     return next_entry(start, true);
