@@ -7,6 +7,7 @@
 #include "application.h"
 #include "buildcfg.h"
 #include "config.h"
+#include "fetcher.h"
 #include "imagelist.h"
 #include "info.h"
 #include "keybind.h"
@@ -74,8 +75,8 @@ struct viewer {
     int slideshow_fd;      ///< Slideshow timer
     size_t slideshow_time; ///< Slideshow image display time (seconds)
 
-    bool info_timedout; ///< Indicates info block shouldn't be displayed anymore
-    int info_timeout_fd; ///< Info timer
+    size_t history; ///< Max number of cached images
+    size_t preload; ///< Max number of images to preload
 };
 
 /** Global viewer context. */
@@ -90,7 +91,7 @@ static void fixup_position(bool force)
     const ssize_t wnd_width = ui_get_width();
     const ssize_t wnd_height = ui_get_height();
 
-    const struct pixmap* img = &loader_current_image()->frames[ctx.frame].pm;
+    const struct pixmap* img = &fetcher_current()->frames[ctx.frame].pm;
     const ssize_t img_width = ctx.scale * img->width;
     const ssize_t img_height = ctx.scale * img->height;
 
@@ -177,7 +178,7 @@ static void move_image(bool horizontal, bool positive, const char* params)
  */
 static void rotate_image(bool clockwise)
 {
-    struct image* img = loader_current_image();
+    struct image* img = fetcher_current();
     const struct pixmap* pm = &img->frames[ctx.frame].pm;
     const ssize_t diff = (ssize_t)pm->width - pm->height;
     const ssize_t shift = (ctx.scale * diff) / 2;
@@ -196,7 +197,7 @@ static void rotate_image(bool clockwise)
  */
 static void scale_image(enum fixed_scale sc)
 {
-    const struct image* img = loader_current_image();
+    const struct image* img = fetcher_current();
     const struct pixmap* pm = &img->frames[ctx.frame].pm;
     const size_t wnd_width = ui_get_width();
     const size_t wnd_height = ui_get_height();
@@ -232,6 +233,7 @@ static void scale_image(enum fixed_scale sc)
     ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
 
     fixup_position(true);
+    info_update(info_scale, "%.0f%%", ctx.scale * 100);
     app_redraw();
 }
 
@@ -267,7 +269,7 @@ static void zoom_image(const char* params)
                 ctx.scale = MAX_SCALE;
             }
         } else {
-            const struct image* img = loader_current_image();
+            const struct image* img = fetcher_current();
             const struct pixmap* pm = &img->frames[ctx.frame].pm;
             const float scale_w = (float)MIN_SCALE / pm->width;
             const float scale_h = (float)MIN_SCALE / pm->height;
@@ -286,6 +288,7 @@ static void zoom_image(const char* params)
         fprintf(stderr, "Invalid zoom operation: \"%s\"\n", params);
     }
 
+    info_update(info_scale, "%.0f%%", ctx.scale * 100);
     app_redraw();
 }
 
@@ -298,7 +301,7 @@ static void animation_ctl(bool enable)
     struct itimerspec ts = { 0 };
 
     if (enable) {
-        const struct image* img = loader_current_image();
+        const struct image* img = fetcher_current();
         const size_t duration = img->frames[ctx.frame].duration;
         enable = (img->num_frames > 1 && duration);
         if (enable) {
@@ -332,14 +335,13 @@ static void slideshow_ctl(bool enable)
  */
 static void reset_state(void)
 {
-    const struct image* img = loader_current_image();
-    const int timeout = info_timeout();
+    const struct image* img = fetcher_current();
+    const size_t total_img = image_list_size();
 
     ctx.frame = 0;
     ctx.img_x = 0;
     ctx.img_y = 0;
     ctx.scale = 0;
-    ctx.info_timedout = false;
     scale_image(ctx.scale_init);
     fixup_position(true);
 
@@ -347,17 +349,9 @@ static void reset_state(void)
     animation_ctl(true);
     slideshow_ctl(ctx.slideshow_enable);
 
-    // Expire info block after timeout
-    if (timeout != 0) {
-        struct itimerspec info_ts = { 0 };
-        if (timeout > 0) {
-            // absolute time in sec
-            info_ts.it_value.tv_sec = timeout;
-        } else if (ctx.slideshow_enable) {
-            // slideshow relative in %
-            info_ts.it_value.tv_sec = ctx.slideshow_time * -timeout / 100;
-        }
-        timerfd_settime(ctx.info_timeout_fd, 0, &info_ts, NULL);
+    info_reset(img);
+    if (total_img) {
+        info_update(info_index, "%zu of %zu", img->index + 1, total_img);
     }
 
     app_redraw();
@@ -369,9 +363,9 @@ static void reset_state(void)
  */
 static bool skip_image(void)
 {
-    size_t index = image_list_skip(loader_current_index());
+    size_t index = image_list_skip(fetcher_current()->index);
 
-    while (index != IMGLIST_INVALID && !loader_get_image(index)) {
+    while (index != IMGLIST_INVALID && !fetcher_open(index)) {
         index = image_list_next_file(index);
     }
 
@@ -385,7 +379,7 @@ static bool skip_image(void)
  */
 static bool next_image(enum action_type direction)
 {
-    size_t index = loader_current_index();
+    size_t index = fetcher_current()->index;
 
     do {
         switch (direction) {
@@ -410,7 +404,7 @@ static bool next_image(enum action_type direction)
             default:
                 break;
         }
-    } while (index != IMGLIST_INVALID && !loader_get_image(index));
+    } while (index != IMGLIST_INVALID && !fetcher_open(index));
 
     if (index == IMGLIST_INVALID) {
         return false;
@@ -428,7 +422,7 @@ static bool next_image(enum action_type direction)
 static void next_frame(bool forward)
 {
     size_t index = ctx.frame;
-    const struct image* img = loader_current_image();
+    const struct image* img = fetcher_current();
 
     if (forward) {
         if (++index >= img->num_frames) {
@@ -441,6 +435,9 @@ static void next_frame(bool forward)
     }
     if (index != ctx.frame) {
         ctx.frame = index;
+        info_update(info_frame, "%zu of %zu", ctx.frame + 1, img->num_frames);
+        info_update(info_image_size, "%zux%zu", img->frames[ctx.frame].pm.width,
+                    img->frames[ctx.frame].pm.height);
         app_redraw();
     }
 }
@@ -448,7 +445,7 @@ static void next_frame(bool forward)
 /**
  * Animation timer event handler.
  */
-static void on_animation_timer(void)
+static void on_animation_timer(__attribute__((unused)) void* data)
 {
     next_frame(true);
     animation_ctl(true);
@@ -457,23 +454,11 @@ static void on_animation_timer(void)
 /**
  * Slideshow timer event handler.
  */
-static void on_slideshow_timer(void)
+static void on_slideshow_timer(__attribute__((unused)) void* data)
 {
     slideshow_ctl(next_image(action_next_file));
 }
 
-/**
- * Info block timer event handler.
- */
-static void on_info_block_timeout(void)
-{
-    // Reset timer to 0, so it won't fire again
-    struct itimerspec info_ts = { 0 };
-    timerfd_settime(ctx.info_timeout_fd, 0, &info_ts, NULL);
-    ctx.info_timedout = true;
-    app_redraw();
-}
-
 /**
  * Show/hide help layer.
  */
@@ -495,7 +480,7 @@ static void switch_help(void)
 
     // get number of bindings
     ctx.help_sz = 0;
-    kb = keybind_all();
+    kb = keybind_get();
     while (kb) {
         if (kb->help) {
             ++ctx.help_sz;
@@ -505,7 +490,7 @@ static void switch_help(void)
     ctx.help = calloc(1, ctx.help_sz * sizeof(*ctx.help));
     if (ctx.help) {
         size_t i = ctx.help_sz - 1;
-        kb = keybind_all();
+        kb = keybind_get();
         while (kb) {
             if (kb->help) {
                 font_render(kb->help, &ctx.help[i--]);
@@ -521,7 +506,7 @@ static void switch_help(void)
  */
 static void draw_image(struct pixmap* wnd)
 {
-    const struct image* img = loader_current_image();
+    const struct image* img = fetcher_current();
     const struct pixmap* img_pm = &img->frames[ctx.frame].pm;
     const size_t width = ctx.scale * img_pm->width;
     const size_t height = ctx.scale * img_pm->height;
@@ -558,10 +543,17 @@ static void draw_image(struct pixmap* wnd)
  */
 static void reload(void)
 {
-    if (loader_reset(loader_current_index(), false) == ldr_success) {
-        app_status("Image reloaded");
+    const size_t index = fetcher_current()->index;
+
+    if (fetcher_reset(index, false)) {
+        if (index == fetcher_current()->index) {
+            info_update(info_status, "Image reloaded");
+        } else {
+            info_update(info_status, "Unable to update, open next file");
+        }
         reset_state();
     } else {
+        printf("No more images to view, exit\n");
         app_exit(0);
     }
 }
@@ -576,33 +568,13 @@ static void redraw(void)
         return;
     }
 
-    info_update(ctx.frame, ctx.scale);
     draw_image(window);
-
-    // put text info blocks on window surface
-    if (!ctx.info_timedout) {
-        const enum text_position pos[] = {
-            text_top_left,
-            text_top_right,
-            text_bottom_left,
-            text_bottom_right,
-        };
-        for (size_t i = 0; i < ARRAY_SIZE(pos); ++i) {
-            const size_t lines_num = info_height(pos[i]);
-            if (lines_num) {
-                const struct text_keyval* lines = info_lines(pos[i]);
-                text_print_keyval(window, pos[i], lines, lines_num);
-            }
-        }
-    }
+    info_print(window);
 
     if (ctx.help) {
         text_print_lines(window, text_center, ctx.help, ctx.help_sz);
     }
 
-    // reset one-time rendered notification message
-    app_status(NULL);
-
     ui_draw_commit();
 }
 
@@ -622,12 +594,12 @@ static void on_resize(void)
  */
 static void on_keyboard(xkb_keysym_t key, uint8_t mods)
 {
-    const struct keybind* kb = keybind_get(key, mods);
+    const struct keybind* kb = keybind_find(key, mods);
 
     if (!kb) {
         char* name = keybind_name(key, mods);
         if (name) {
-            app_status("Key %s is not bound", name);
+            info_update(info_status, "Key %s is not bound", name);
             free(name);
             app_redraw();
         }
@@ -675,7 +647,7 @@ static void on_keyboard(xkb_keysym_t key, uint8_t mods)
                 ui_toggle_fullscreen();
                 break;
             case action_mode:
-                app_switch_mode();
+                app_switch_mode(fetcher_current()->index);
                 break;
             case action_step_left:
                 move_image(true, true, action->params);
@@ -699,30 +671,31 @@ static void on_keyboard(xkb_keysym_t key, uint8_t mods)
                 rotate_image(true);
                 break;
             case action_flip_vertical:
-                image_flip_vertical(loader_current_image());
+                image_flip_vertical(fetcher_current());
                 app_redraw();
                 break;
             case action_flip_horizontal:
-                image_flip_horizontal(loader_current_image());
+                image_flip_horizontal(fetcher_current());
                 app_redraw();
                 break;
             case action_antialiasing:
                 ctx.antialiasing = !ctx.antialiasing;
-                app_status("Anti-aliasing %s", ctx.antialiasing ? "on" : "off");
+                info_update(info_status, "Anti-aliasing %s",
+                            ctx.antialiasing ? "on" : "off");
                 app_redraw();
                 break;
             case action_reload:
                 reload();
                 break;
             case action_info:
-                info_set_mode(action->params);
+                info_switch(action->params);
                 app_redraw();
                 break;
             case action_exec:
-                app_execute(action->params, loader_current_image()->source);
+                app_execute(action->params, fetcher_current()->source);
                 break;
             case action_status:
-                app_status("%s", action->params);
+                info_update(info_status, "%s", action->params);
                 app_redraw();
                 break;
             case action_exit:
@@ -763,14 +736,8 @@ static enum config_status load_config(const char* key, const char* value)
 {
     enum config_status status = cfgst_invalid_value;
 
-    if (strcmp(key, VIEWER_CFG_ANTIALIASING) == 0) {
-        if (config_to_bool(value, &ctx.antialiasing)) {
-            status = cfgst_ok;
-        }
-    } else if (strcmp(key, VIEWER_CFG_SCALE) == 0) {
-        const ssize_t index = str_index(scale_names, value, 0);
-        if (index >= 0) {
-            ctx.scale_init = index;
+    if (strcmp(key, VIEWER_CFG_WINDOW) == 0) {
+        if (config_to_color(value, &ctx.window_bkg)) {
             status = cfgst_ok;
         }
     } else if (strcmp(key, VIEWER_CFG_TRANSPARENCY) == 0) {
@@ -780,14 +747,20 @@ static enum config_status load_config(const char* key, const char* value)
         } else if (config_to_color(value, &ctx.image_bkg)) {
             status = cfgst_ok;
         }
-    } else if (strcmp(key, VIEWER_CFG_BACKGROUND) == 0) {
-        if (config_to_color(value, &ctx.window_bkg)) {
+    } else if (strcmp(key, VIEWER_CFG_SCALE) == 0) {
+        const ssize_t index = str_index(scale_names, value, 0);
+        if (index >= 0) {
+            ctx.scale_init = index;
             status = cfgst_ok;
         }
     } else if (strcmp(key, VIEWER_CFG_FIXED) == 0) {
         if (config_to_bool(value, &ctx.fixed)) {
             status = cfgst_ok;
         }
+    } else if (strcmp(key, VIEWER_CFG_ANTIALIASING) == 0) {
+        if (config_to_bool(value, &ctx.antialiasing)) {
+            status = cfgst_ok;
+        }
     } else if (strcmp(key, VIEWER_CFG_SLIDESHOW) == 0) {
         if (config_to_bool(value, &ctx.slideshow_enable)) {
             status = cfgst_ok;
@@ -798,6 +771,18 @@ static enum config_status load_config(const char* key, const char* value)
             ctx.slideshow_time = num;
             status = cfgst_ok;
         }
+    } else if (strcmp(key, VIEWER_CFG_HISTORY) == 0) {
+        ssize_t num;
+        if (str_to_num(value, 0, &num, 0) && num >= 0 && num < 1024) {
+            ctx.history = num;
+            status = cfgst_ok;
+        }
+    } else if (strcmp(key, VIEWER_CFG_PRELOAD) == 0) {
+        ssize_t num;
+        if (str_to_num(value, 0, &num, 0) && num >= 0 && num < 1024) {
+            ctx.preload = num;
+            status = cfgst_ok;
+        }
     } else {
         status = cfgst_invalid_key;
     }
@@ -815,41 +800,36 @@ void viewer_create(void)
     ctx.slideshow_enable = false;
     ctx.slideshow_fd = -1;
     ctx.slideshow_time = 3;
-    ctx.info_timedout = false;
-    ctx.info_timeout_fd = -1;
+    ctx.history = 1;
+    ctx.preload = 1;
 
     // register configuration loader
-    config_add_loader(GENERAL_CONFIG_SECTION, load_config);
+    config_add_loader(VIEWER_CFG_SECTION, load_config);
 }
 
-void viewer_init(void)
+void viewer_init(struct image* image)
 {
     // setup animation timer
     ctx.animation_fd =
         timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC | TFD_NONBLOCK);
     if (ctx.animation_fd != -1) {
-        app_watch(ctx.animation_fd, on_animation_timer);
+        app_watch(ctx.animation_fd, on_animation_timer, NULL);
     }
 
     // setup slideshow timer
     ctx.slideshow_fd =
         timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC | TFD_NONBLOCK);
     if (ctx.slideshow_fd != -1) {
-        app_watch(ctx.slideshow_fd, on_slideshow_timer);
+        app_watch(ctx.slideshow_fd, on_slideshow_timer, NULL);
     }
 
-    // setup info block timer
-    if (info_timeout() != 0) {
-        ctx.info_timeout_fd =
-            timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC | TFD_NONBLOCK);
-        if (ctx.info_timeout_fd != -1) {
-            app_watch(ctx.info_timeout_fd, on_info_block_timeout);
-        }
-    }
+    fetcher_init(image, ctx.history, ctx.preload);
 }
 
 void viewer_destroy(void)
 {
+    fetcher_destroy();
+
     for (size_t i = 0; i < ctx.help_sz; i++) {
         free(ctx.help[i].data);
     }
@@ -859,9 +839,6 @@ void viewer_destroy(void)
     if (ctx.slideshow_fd != -1) {
         close(ctx.slideshow_fd);
     }
-    if (ctx.info_timeout_fd != -1) {
-        close(ctx.info_timeout_fd);
-    }
 }
 
 void viewer_handle(const struct event* event)
@@ -883,8 +860,14 @@ void viewer_handle(const struct event* event)
             on_drag(event->param.drag.dx, event->param.drag.dy);
             break;
         case event_activate:
-            reset_state();
-            app_redraw();
+            if (fetcher_reset(event->param.activate.index, false)) {
+                reset_state();
+            } else {
+                app_exit(0);
+            }
+            break;
+        case event_load:
+            fetcher_attach(event->param.load.image, event->param.load.index);
             break;
     }
 }
